#include <asm/io.h>
/*

 LM75 reader.

 Written by Philip Edelbrock, June 1998

*/

// #define DEBUG
#define MAX_TIMEOUT 50

//#define PIIX4_smba 0x0FCB0
#define SMBHSTSTS PIIX4_smba
#define SMBHSLVSTS PIIX4_smba + 1
#define SMBHSTCNT PIIX4_smba + 2
#define SMBHSTCMD PIIX4_smba + 3
#define SMBHSTADD PIIX4_smba + 4
#define SMBHSTDAT0 PIIX4_smba + 5
#define SMBHSTDAT1 PIIX4_smba + 6


// ****CHANGE THIS SO IT IS VALID FOR YOUR MACHINE!!!****
long PIIX4_smba = 0x0FCB0;


static int read_smbus_word(int addr);

int write_smbus_word(int addr, char hi, char lo);

static void debug_status();

unsigned char  PIIX4_bus,PIIX4_devfn;


void debug_status() {

#ifdef DEBUG
    printf("SMBHSTSTS  :0x%X\n",inb_p(PIIX4_smba));
    printf("SMBHSLVSTS :0x%X\n",inb_p(PIIX4_smba + 1));
    printf("SMBHSTCNT  :0x%X\n",inb_p(PIIX4_smba + 2));
    printf("SMBHSTCMD  :0x%X\n",inb_p(PIIX4_smba + 3));
    printf("SMBHSTADD  :0x%X\n",inb_p(PIIX4_smba + 4));
    printf("SMBHSTDAT0 :0x%X\n",inb_p(PIIX4_smba + 5));
    printf("SMBHSTDAT1 :0x%X\n",inb_p(PIIX4_smba + 6));
#endif
}

/*

int  read_smbus_word(int addr)

	Read the word from device at Address 'addr', and return it.
	Return -1 if there is a timeout for this device address.

*/


int read_smbus_word(int addr) {


int HOST_BUSY;
long timeout=0;
int result=0;

// Setup protocol type, address, device command, data, and control bits
#ifdef DEBUG
    printf("entering read_smbus_byte\n");
#endif
    outb_p( 0x00, SMBHSTDAT0);
    outb_p( 0x00, SMBHSTDAT1);
    outb_p( 0x0C, SMBHSTCNT); // Read word
    debug_status();
    if (inb_p(SMBHSTSTS) != 0x00) {
#ifdef DEBUG
        printf("SMBus busy! (SMBHSTSTS: 0x%X)\n",inb(SMBHSTSTS));
        printf("attempting to reset SMBus...\n");
#endif
        outb_p( inb(SMBHSTSTS), SMBHSTSTS);
        if (!(inb_p(SMBHSTSTS))) {
#ifdef DEBUG

                printf("Reset successful! (SMBHSTSTS: 0x%X)\n",inb_p(SMBHSTSTS));
#endif
	}
        else {
#ifdef DEBUG
                printf("Reset unsuccessful! (SMBHSTSTS: 0x%X)\n",inb_p(SMBHSTSTS));
#endif
	}
    }
    if (inb(SMBHSTSTS) & 0x01) {
#ifdef DEBUG
        printf("SMBus busy! (Slave port busy SMBHSTSTS: 0x%X)",inb_p(SMBHSTSTS));
#endif
    }

    outb_p( (addr << 1) | 0x01, SMBHSTADD); // address of possible thermal sensors are from 1001000x to 1001111x ( x = r/w bit )
    

// Set the start bit
    outb_p(inb(SMBHSTCNT) | 0x040, SMBHSTCNT); // start the transaction by setting bit 6

// Wait for a fraction of a second!
   usleep(100);

// Pole Host_busy bit
   HOST_BUSY=inb_p(SMBHSTSTS) & 0x01;
#ifdef DEBUG
   printf("Host busy status: %X\n",HOST_BUSY);
#endif
   while (((inb_p(SMBHSTSTS) & 0x01)) & (timeout < MAX_TIMEOUT)) {
// busy sleep [not good for final implementation]
	usleep(50);
   }
   if (! (inb(SMBHSTSTS) & 0x01)) {
#ifdef DEBUG
        printf("(Timeout count: %i)\n",timeout);
#endif
        result=(inb(SMBHSTDAT0) << 8) | inb(SMBHSTDAT1);
#ifdef DEBUG
        printf("Result: 0x%X\n",result);
#endif
   } else {
#ifdef DEBUG
        printf("Timeout!\n");
#endif
   }
   debug_status();

   if (inb_p(SMBHSTSTS) == 0x04)  { // Sense for no device at the address
	result=-1;
#ifdef DEBUG
	printf("Nothing at address!\n");
#endif
   }
   return result;
}

int write_smbus_word(int addr, char hi, char lo) {

int HOST_BUSY;
long timeout=0;
int result=0;

// Setup protocol type, address, device command, data, and control bits
#ifdef DEBUG
    printf("entering write_smbus_byte\n");
#endif
    outb_p(lo, SMBHSTDAT0);
    outb_p(hi, SMBHSTDAT1);
    debug_status();
    if (inb_p(SMBHSTSTS) != 0x00) {
#ifdef DEBUG
        printf("SMBus busy! (SMBHSTSTS: 0x%X)\n",inb(SMBHSTSTS));
        printf("attempting to reset SMBus...\n");
#endif
        outb_p( inb(SMBHSTSTS), SMBHSTSTS);
        if (!(inb_p(SMBHSTSTS))) {
#ifdef DEBUG

                printf("Reset successful! (SMBHSTSTS: 0x%X)\n",inb_p(SMBHSTSTS));
#endif
	}
        else {
#ifdef DEBUG
                printf("Reset unsuccessful! (SMBHSTSTS: 0x%X)\n",inb_p(SMBHSTSTS));
#endif
	}
    }
    if (inb(SMBHSTSTS) & 0x01) {
#ifdef DEBUG
        printf("SMBus busy! (Slave port busy SMBHSTSTS: 0x%X)",inb_p(SMBHSTSTS));
#endif
    }

    outb_p( (addr << 1) | 0x01, SMBHSTADD); // address of possible thermal sensors are from 1001000x to 1001111x ( x = r/w bit )
    

// Set the start bit
    outb_p(inb(SMBHSTCNT) | 0x040, SMBHSTCNT); // start the transaction by setting bit 6

// Wait for a fraction of a second!
   usleep(100);

// Pole Host_busy bit
   HOST_BUSY=inb_p(SMBHSTSTS) & 0x01;
#ifdef DEBUG
   printf("Host busy status: %X\n",HOST_BUSY);
#endif
   while (((inb_p(SMBHSTSTS) & 0x01)) & (timeout < MAX_TIMEOUT)) {
// busy sleep [not good for final implementation]
	usleep(50);
   }
   if (! (inb(SMBHSTSTS) & 0x01)) {
#ifdef DEBUG
        printf("(Timeout count: %i)\n",timeout);
#endif
        result=inb(SMBHSTDAT0);
#ifdef DEBUG
        printf("Result: 0x%X\n",result);
#endif
   } else {
#ifdef DEBUG
        printf("Timeout!\n");
#endif
   }
   debug_status();

   if (inb_p(SMBHSTSTS) == 0x04)  { // Sense for no device at the address
	result=-1;
#ifdef DEBUG
	printf("Nothing at address!\n");
#endif
   }
   return result;
}


void init()
{  
/*
    int ret = 0;
    int temp;

    printf("\n");
   
    if (pcibios_present() == 0) {
        
        printf("Error: No PCI-bus found!\n");
        ret = 1;
    
    } else {
    
        if(pcibios_find_device(PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_INTEL_82371AB_0, 0,
                               &PIIX4_bus, &PIIX4_devfn)) {
     
            printf("Error: Cant detect PIIX4!\n");
            ret = 1;

        } else {

            pcibios_read_config_word(PIIX4_bus, PIIX4_devfn | 3, 0x90, &PIIX4_smba);
            PIIX4_smba &= 0xfff0;
   
            request_region(PIIX4_smba, 256, "acops");
        }
   
    }
*/

   ioperm(PIIX4_smba,8,1);
//   printf("I/O Permissions failed! (Must be root!)\n");
   iopl(3);
//        exit();
}



void cleanup ()
{
 /*  if (PIIX4_smba)
       release_region(PIIX4_smba, 256); */
//   printf("Exit\n");
}

int main(int argc, char *argv[]) {

int i,value;

  init();
// Probe each possible valid LM75 address and report the temp for each found
  for (i=0x048; i<=0x04F; i++) { 
//  for (i=0x04C; i<=0x04C; i++) { 
	  value=read_smbus_word(i) >> 7;
#ifdef DEBUG
	  printf("Result: 0x%X\n",value);
#endif
	  if (value != -1) {
		printf("LM75 at address 0x%X reports %.1f degrees C\n",i,value * 0.5);
	  }
  }
  cleanup();
  return 0;
}
